import { Placement, ModifierPhases } from "./enums";
export declare type JQueryWrapper = Array<HTMLElement> & {
    jquery: string;
};
export declare type Obj = {
    [key: string]: any;
};
export declare type Rect = {
    width: number;
    height: number;
    x: number;
    y: number;
};
export declare type Offsets = {
    y: number;
    x: number;
};
export declare type PositioningStrategy = "absolute" | "fixed";
export declare type StateMeasures = {
    reference: Rect;
    popper: Rect;
};
export declare type StateOffsets = {
    popper: Offsets;
    arrow?: Offsets;
};
export declare type State = {
    elements: {
        reference: Element | VirtualElement;
        popper: HTMLElement;
        arrow?: HTMLElement;
    };
    options: Options;
    placement: Placement;
    strategy: PositioningStrategy;
    orderedModifiers: Array<Modifier<any>>;
    measures: StateMeasures;
    scrollParents: {
        reference: Array<Element>;
        popper: Array<Element>;
    };
    styles: {
        [key: string]: Partial<CSSStyleDeclaration>;
    };
    attributes: {
        [key: string]: {
            [key: string]: string | boolean;
        };
    };
    modifiersData: {
        [key: string]: any;
    };
    reset: boolean;
};
export declare type Instance = {
    destroy: () => void;
    forceUpdate: () => void;
    update: () => Promise<void>;
    setOptions: (options: Partial<Options>) => void;
};
export declare type ModifierArguments<Options extends Obj> = {
    state: Partial<State>;
    instance: Instance;
    options: Partial<Options>;
    name: string;
};
export declare type Modifier<Options> = {
    name: string;
    enabled: boolean;
    phase: ModifierPhases;
    requires?: Array<string>;
    optionallyRequires?: Array<string>;
    fn: (arg0: ModifierArguments<Options>) => State;
    onLoad?: (arg0: ModifierArguments<Options>) => State | null;
    onDestroy?: (arg0: ModifierArguments<Options>) => void;
    options?: Obj;
    data?: Obj;
};
export declare type EventListeners = {
    scroll: boolean;
    resize: boolean;
};
export declare type Options = {
    placement: Placement;
    modifiers: Array<Partial<Modifier<any>>>;
    strategy: PositioningStrategy;
};
export declare type UpdateCallback = (arg0: State) => void;
export declare type ClientRectObject = {
    x: number;
    y: number;
    top: number;
    left: number;
    right: number;
    bottom: number;
    width: number;
    height: number;
};
export declare type PaddingObject = {
    top: number;
    left: number;
    right: number;
    bottom: number;
};
export declare type Padding = number | Partial<PaddingObject>;
export declare type VirtualElement = {
    contextElement: HTMLElement;
    getBoundingClientRect: () => ClientRect | DOMRect;
};
