import getBoundingClientRect from "./getBoundingClientRect.js";
import listScrollParents from "./listScrollParents.js";
import getScrollSum from "./getScrollSum.js";
import getOffsetParent from "./getOffsetParent.js";
import unwrapVirtualElement from "./unwrapVirtualElement.js";
import isTableElement from "./isTableElement.js";
import getTrueOffsetParent from "./getTrueOffsetParent.js";
import { isElement } from "./instanceOf.js"; // Returns the composite rect of an element relative to its offsetParent.
// Composite means it takes into account transforms as well as layout.

export default (function (element, isFixed) {
  if (isFixed === void 0) {
    isFixed = false;
  }

  var unwrappedElement = unwrapVirtualElement(element);
  var rect = getBoundingClientRect(element);
  var scrollParents = listScrollParents(unwrappedElement);
  var offsetParent = getOffsetParent(unwrappedElement);
  var offsetParentRect = isElement(offsetParent) && !isFixed ? getBoundingClientRect(offsetParent) : {
    left: 0,
    top: 0
  };
  var trueOffsetParent = getTrueOffsetParent(unwrappedElement); // We want all the scrolling containers only up to and including the
  // offsetParent

  var relevantScrollParents = trueOffsetParent && isTableElement(trueOffsetParent) ? scrollParents : scrollParents.slice(0, Math.max(0, scrollParents.indexOf(offsetParent)) + 1);
  var scrollSum = getScrollSum(relevantScrollParents);
  var offsetParentScrollSum = getScrollSum(isFixed ? [offsetParent] : []);
  var width = rect.width;
  var height = rect.height;
  var x = rect.left + scrollSum.scrollLeft - offsetParentScrollSum.scrollLeft - offsetParentRect.left;
  var y = rect.top + scrollSum.scrollTop - offsetParentScrollSum.scrollTop - offsetParentRect.top;
  return {
    width: width,
    height: height,
    x: x,
    y: y
  };
});