// This modifier takes the Popper state and prepares some StyleSheet properties
// that can be applied to the popper element to make it render in the expected position.
// Round the offsets to the nearest suitable subpixel based on the DPR
var roundOffsets = function roundOffsets(_ref) {
  var x = _ref.x,
      y = _ref.y;
  var dpr = window.devicePixelRatio;
  return {
    x: Math.round(x * dpr) / dpr || 0,
    y: Math.round(y * dpr) / dpr || 0
  };
};

export var mapToStyles = function mapToStyles(_ref2) {
  var offsets = _ref2.offsets,
      position = _ref2.position,
      gpuAcceleration = _ref2.gpuAcceleration;

  var _roundOffsets = roundOffsets(offsets),
      x = _roundOffsets.x,
      y = _roundOffsets.y;

  var hasX = offsets.hasOwnProperty('x');
  var hasY = offsets.hasOwnProperty('y'); // Layer acceleration can disable subpixel rendering which causes slightly
  // blurry text on low PPI displays.
  // Zooming can change the DPR, but it seems to report a value that will
  // cleanly divide the values into the appropriate subpixels.

  if (gpuAcceleration === false || window.devicePixelRatio < 2) {
    return {
      top: hasY ? y + "px" : '',
      left: hasX ? x + "px" : '',
      transform: '',
      position: position
    };
  } else {
    return {
      top: hasY ? '0' : '',
      left: hasX ? '0' : '',
      transform: "translate3d(" + x + "px, " + y + "px, 0)",
      position: position
    };
  }
};
export function computeStyles(_ref3) {
  var state = _ref3.state,
      options = _ref3.options;
  var _options$gpuAccelerat = options.gpuAcceleration,
      gpuAcceleration = _options$gpuAccelerat === void 0 ? true : _options$gpuAccelerat; // popper offsets are always available

  state.styles.popper = Object.assign({}, state.styles.popper, {}, mapToStyles({
    offsets: state.modifiersData.popperOffsets,
    position: state.options.strategy,
    gpuAcceleration: gpuAcceleration
  })); // arrow offsets may not be available

  if (state.modifiersData.arrow != null) {
    state.styles.arrow = Object.assign({}, state.styles.arrow, {}, mapToStyles({
      offsets: state.modifiersData.arrow,
      position: 'absolute',
      gpuAcceleration: gpuAcceleration
    }));
  }

  state.attributes.popper = Object.assign({}, state.attributes.popper, {
    'data-popper-placement': state.placement
  });
  return state;
}
export default {
  name: 'computeStyles',
  enabled: true,
  phase: 'afterMain',
  fn: computeStyles,
  data: {}
};